import {
  Metadata, Loader, IDictionary,
} from '@zeedhi/core';

import {
  GridEditable, Card, Form, DialogService,
} from '@zeedhi/common';

export class Ger22000ApresentacaoProdutoController {
  public gridEditController = Loader.getInstance('GridEditableController');

  public crudController = Loader.getInstance('CrudController');
  
  public utilities = Loader.getInstance('Utilities');
  
  private apresentacaoIsVisible = false;

  public openProduct() {
    const produtoApresentacao: GridEditable = Metadata.getInstance('gridApresentacao');
    const form = Metadata.getInstance<Form>('formFilter').value;
    const produto = form.CDPRODUTO.CDPRODUTO;
    
    if (produto !== null) {
      const conditions = [
        {
          name: 'CDPRODUTO',
          operator: '=',
          value: produto,
        },
      ];
      produtoApresentacao.datasource.filter = conditions;
      produtoApresentacao.datasource.get();
    } else produtoApresentacao.datasource.data = [];
  }

  private cardApresentacaoIsVisible() {
    const card = Metadata.getInstance<Card>('cardApresentacao');
    try {
      if (!this.gridEditController.newRow) {
        card.isVisible = true;
        this.apresentacaoIsVisible = true;
      }
    } catch (Error) {
      card.isVisible = false;
      this.apresentacaoIsVisible = false;
    }
  }
  
  public salvaApresentacao() {
    const gridApresentacao: GridEditable = Metadata.getInstance('gridApresentacao');
    const form = Metadata.getInstance<Form>('formFilter').value;
    const editedRows: IDictionary = gridApresentacao.getEditedRows();
    // editedRows[0].SGUNIDADE = form.CDPRODUTO.SGUNIDADE;
    editedRows[0].CDPRODUTO = form.CDPRODUTO.CDPRODUTO;
    
    this.crudController.saveRows(editedRows, 'apreprod', gridApresentacao).then(() => {
      this.gridEditController.defaultState(gridApresentacao);
      gridApresentacao.datasource.get();
      this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }
  
  public async deleteCheckedRows() {
    const gridApresentacao: GridEditable = Metadata.getInstance('gridApresentacao');
    const checkedRows = gridApresentacao.selectedRows;
    
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'confirmDeleteNutri',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteApresentacao() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteApresentacao() {
    const gridApresentacao: GridEditable = Metadata.getInstance('gridApresentacao');
    const checkedRows = gridApresentacao.selectedRows;
    const dataSourceName = '/apreprod';
    this.crudController.deleteRows(checkedRows, dataSourceName).then(() => {
      gridApresentacao.cancelEditedRows();
      gridApresentacao.datasource.get();
      gridApresentacao.selectedRows = [];
      DialogService.hide();
      
      this.utilities.showAlert('alertError', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }
  
  public closeCardProduto() {
    const card = Metadata.getInstance<Card>('cardApresentacao');
    
    card.isVisible = false;
    this.apresentacaoIsVisible = false;
  }
  
  get gridColsReturnValue(): number {
    return this.apresentacaoIsVisible ? 6 : 12;
  }
  
  public apresentacaoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDAPRERECE',
      },
      'NMAPRERECE',
    ],
    formatterDataText: [
      {
        name: 'CDAPRERECE',
      },
      'NMAPRERECE',
    ],
    dataValueOut: [
      {
        column: 'CDAPRERECE',
        columnOnGrid: 'CDAPRERECE',
      },
      {
        column: 'NMAPRERECE',
        columnOnGrid: 'NMAPRERECE',
      },
    ],
    dataValue: 'CDAPRERECE',
    dense: true,
    datasource: {
      uniqueKey: 'CDAPRERECE',
      route: '/aprerece',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
}
